use crate::{
    components::{
        monster::{self, ticks_for_kind},
        Monster, Position, Renderable,
    },
    map::{Map, TileType},
    resources::{Clock, Stats},
};
use bracket_lib::{prelude::*, random::RandomNumberGenerator};
use specs::prelude::*;

pub struct MonsterMotion {}

#[allow(clippy::type_complexity)]
impl<'a> System<'a> for MonsterMotion {
    type SystemData = (
        Entities<'a>,
        ReadExpect<'a, Clock>,
        ReadExpect<'a, Point>,
        WriteExpect<'a, Map>,
        WriteExpect<'a, Stats>,
        WriteStorage<'a, Monster>,
        WriteStorage<'a, Position>,
        WriteStorage<'a, Renderable>,
    );

    fn run(
        &mut self,
        (
            entities,
            clock,
            player_pos,
            mut map,
            mut stats,
            mut monster,
            mut position,
            mut renderable,
        ): Self::SystemData,
    ) {
        for (entity, monster, position, renderable) in
            (&entities, &mut monster, &mut position, &mut renderable).join()
        {
            if clock.has_ticked {
                monster.ticks_until_move -= 1;
                if monster.ticks_until_move <= 0 {
                    // Change glyph
                    let mut rng = RandomNumberGenerator::new();
                    if let Some(glyph) =
                        rng.random_slice_entry(&monster::glyphs_for_kind(monster.kind))
                    {
                        renderable.glyph = *glyph;
                    }

                    // Move monster
                    let (mut delta_x, mut delta_y) = (0, 0);
                    if player_pos.x < position.x {
                        delta_x = -1;
                    }
                    if player_pos.x > position.x {
                        delta_x = 1;
                    }
                    if player_pos.y < position.y {
                        delta_y = -1;
                    }
                    if player_pos.y > position.y {
                        delta_y = 1;
                    }
                    let destination = Point {
                        x: position.x + delta_x,
                        y: position.y + delta_y,
                    };

                    if destination == *player_pos {
                        // TODO: Sound
                        stats.gems -= 1;
                        let _ = entities.delete(entity);
                    }

                    match map.get_tile_at(destination) {
                        TileType::Wall => {}
                        TileType::BreakableWall => {
                            // TODO: Sound
                            map.set_tile_at(destination, TileType::Floor);
                            let _ = entities.delete(entity);
                        }
                        TileType::Floor => {
                            position.x = destination.x;
                            position.y = destination.y;
                        }
                    }

                    monster.ticks_until_move = ticks_for_kind(monster.kind);
                }
            }
        }
    }
}
